'''
Created on 2015/7/30

:author: hubo
'''
from pprint import pprint
from vlcp.server import Server
from vlcp.event import Client, RoutineContainer
from vlcp.protocol.openflow import Openflow, OpenflowConnectionStateEvent, Openflow
from vlcp.protocol.openflow import common
import sys
import logging
from vlcp.utils.namedstruct import NamedStruct

of_proto = Openflow((common.OFP10_VERSION,))

class MainRoutine(RoutineContainer):
    def main(self):
        connected = OpenflowConnectionStateEvent.createMatcher()
        yield (connected,)
        pprint(common.dump(self.event.connection.openflow_featuresreply))
        connection = self.event.connection
        currdef = connection.openflowdef
        for m in of_proto.batch((currdef.nx_set_flow_format.new(format=currdef.NXFF_NXM),), connection, self):
            yield m
        for msg in self.openflow_reply:
            pprint(common.dump(msg))
        for m in of_proto.querymultipart(currdef.ofp_stats_request.new(
                type = currdef.OFPST_DESC), connection, self):
            yield m
        for msg in self.openflow_reply:
            pprint(common.dump(msg))
        req = currdef.ofp_flow_stats_request.new(
                table_id = currdef.OFPTT_ALL,
                out_port = currdef.OFPP_NONE,
                match = currdef.ofp_match.new(wildcards=currdef.OFPFW_ALL))
        for m in of_proto.querymultipart(req, connection, self):
            yield m
        for msg in self.openflow_reply:
            pprint(common.dump(msg, dumpextra = True, typeinfo = common.DUMPTYPE_FLAT))
        req = currdef.nx_flow_stats_request.new(table_id = currdef.OFPTT_ALL, out_port = currdef.OFPP_NONE)
        for m in of_proto.querymultipart(req, connection, self):
            yield m
        for msg in self.openflow_reply:
            pprint(common.dump(msg, dumpextra = True, typeinfo = common.DUMPTYPE_FLAT))
        req = currdef.ofp_msg.new()
        req.header.type = currdef.OFPT_GET_CONFIG_REQUEST
        for m in of_proto.querywithreply(req, connection, self):
            yield m
        pprint(common.dump(self.openflow_reply))
        for m in mgt_conn.shutdown(False):
            yield m

if __name__ == '__main__':
    logging.basicConfig()
    s = Server()
    #s.scheduler.logger.setLevel(logging.DEBUG)
    #of_proto._logger.setLevel(logging.DEBUG)
    #of_proto.debugging = True
    #NamedStruct._logger.setLevel(logging.DEBUG)
    bridge = sys.argv[1]
    routine = MainRoutine(s.scheduler)
    routine.start()
    mgt_conn = Client('unix:/var/run/openvswitch/' + bridge + '.mgmt', of_proto, s.scheduler)
    mgt_conn.start()
    s.serve()
    



'''
Created on 2016/3/8

:author: hubo
'''
from vlcp.config import defaultconfig
from vlcp.service.connection.tcpserver import TcpServerBase
from vlcp.protocol.redis import Redis
from vlcp.server.module import api
from vlcp.utils.redisclient import RedisClient
from vlcp.event.runnable import RoutineContainer
from zlib import compress, decompress, error as zlib_error
import pickle
try:
    import cPickle
except ImportError:
    pass
import json
from vlcp.utils.jsonencoder import encode_default, decode_object
import itertools
try:
    from itertools import izip
except ImportError:
    def izip(*args, **kwargs):
        return iter(zip(*args, **kwargs))

import random

class RedisWriteConflictException(Exception):
    pass

@defaultconfig
class RedisDB(TcpServerBase):
    '''
    Create redis clients to connect to redis server
    '''
    _default_url = 'tcp://localhost/'
    _default_db = None
    _default_serialize = 'json'
    _default_deflate = True
    _default_pickleversion = 'default'
    _default_cpickle = True
    _default_maxretry = 16
    _default_maxspin = 10
    client = True
    def __init__(self, server):
        self._redis_clients = {}
        TcpServerBase.__init__(self, server, Redis)
        if self.serialize == 'pickle' or self.serialize == 'cpickle' or self.serialize == 'cPickle':
            if self.serialize == 'pickle':
                if not self.cpickle:
                    p = pickle
                else:
                    p = cPickle
            else:
                p = cPickle
            if self.pickleversion is None or self.pickleversion == 'default':
                pickleversion = None
            elif self.pickleversion == 'highest':
                pickleversion = p.HIGHEST_PROTOCOL
            else:
                pickleversion = self.pickleversion
            if self.deflate:
                def _encode(obj):
                    return compress(p.dumps(obj, pickleversion), 1)
            else:
                def _encode(obj):
                    return p.dumps(obj, pickleversion)
            self._encode = _encode
            if self.deflate:
                def _decode(data):
                    if data is None:
                        return None
                    elif isinstance(data, Exception):
                        raise data
                    else:
                        try:
                            return p.loads(decompress(data))
                        except zlib_error:
                            return p.loads(data)
            else:
                def _decode(data):
                    if data is None:
                        return None
                    elif isinstance(data, Exception):
                        raise data
                    else:
                        return p.loads(data)
            self._decode = _decode
        else:
            if self.deflate:
                def _encode(obj):
                    return compress(json.dumps(obj, default=encode_default).encode('utf-8'), 1)
                self._encode = _encode
                def _decode(data):
                    if data is None:
                        return None
                    elif isinstance(data, Exception):
                        raise data
                    else:
                        try:
                            data = decompress(data)
                        except zlib_error:
                            pass
                        if not isinstance(data, str) and isinstance(data, bytes):
                            data = data.decode('utf-8')
                        return json.loads(data, object_hook=decode_object)
                self._decode = _decode
            else:
                def _encode(obj):
                    return json.dumps(obj, default=encode_default).encode('utf-8')
                self._encode = _encode
                def _decode(data):
                    if data is None:
                        return None
                    elif isinstance(data, Exception):
                        raise data
                    elif not isinstance(data, str) and isinstance(data, bytes):
                        data = data.decode('utf-8')
                    return json.loads(data, object_hook=decode_object)
                self._decode = _decode                
        self.appendAPI(api(self.getclient),
                       api(self.get, self.apiroutine),
                       api(self.set, self.apiroutine),
                       api(self.delete, self.apiroutine),
                       api(self.mget, self.apiroutine),
                       api(self.mset, self.apiroutine),
                       api(self.update, self.apiroutine),
                       api(self.mupdate, self.apiroutine),
                       api(self.updateall, self.apiroutine),
                       api(self.updateallwithtime, self.apiroutine))
    def _client_class(self, config, protocol, vhost):
        db = getattr(config, 'db', None)
        def _create_client(url, protocol, scheduler = None, key = None, certificate = None, ca_certs = None, bindaddress = None):
            c = RedisClient(url, db, protocol)
            if key:
                c.key = key
            if certificate:
                c.certificate = certificate
            if ca_certs:
                c.ca_certs = ca_certs
            self._redis_clients[vhost] = c
            return c.make_connobj(self.apiroutine)
        return _create_client
    def getclient(self, vhost = ''):
        "Return a tuple of (redisclient, encoder, decoder) for specified vhost"
        return (self._redis_clients.get(vhost), self._encode, self._decode)
    def get(self, key, timeout = None, vhost = ''):
        "Get value from key"
        c = self._redis_clients.get(vhost)
        if c is None:
            raise ValueError('vhost ' + repr(vhost) + ' is not defined')
        if timeout is not None:
            if timeout <= 0:
                for m in c.batch_execute(self.apiroutine, ('MULTI',), 
                                                        ('GET', key),
                                                        ('DEL', key),
                                                        ('EXEC',)):
                    yield m
            else:
                for m in c.batch_execute(self.apiroutine, ('MULTI',),
                                                        ('GET', key),
                                                        ('PEXPIRE', key, int(timeout * 1000)),
                                                        ('EXEC',)):
                    yield m
            r = self.apiroutine.retvalue[3][0]
            if isinstance(r, Exception):
                raise r
            self.apiroutine.retvalue = self._decode(r)
        else:
            for m in c.execute_command(self.apiroutine, 'GET', key):
                yield m
            self.apiroutine.retvalue = self._decode(self.apiroutine.retvalue)
    def set(self, key, value, timeout = None, vhost = ''):
        "Set value to key, with an optional timeout"
        c = self._redis_clients.get(vhost)
        if timeout is None:
            for m in c.execute_command(self.apiroutine, 'SET', key, self._encode(value)):
                yield m
        elif timeout <= 0:
            for m in c.execute_command(self.apiroutine, 'DEL', key):
                yield m
        else:
            for m in c.execute_command(self.apiroutine, 'PSETEX', key, int(timeout * 1000), self._encode(value)):
                yield m
        self.apiroutine.retvalue = None
    def delete(self, key, vhost = ''):
        c = self._redis_clients.get(vhost)
        for m in c.execute_command(self.apiroutine, 'DEL', key):
            yield m
        self.apiroutine.retvalue = None
    def mget(self, keys, vhost = ''):
        "Get multiple values from multiple keys"
        if not keys:
            self.apiroutine.retvalue = []
            raise StopIteration
        c = self._redis_clients.get(vhost)
        for m in c.execute_command(self.apiroutine, 'MGET', *keys):
            yield m
        self.apiroutine.retvalue = [self._decode(r) for r in self.apiroutine.retvalue]
    def mset(self, kvpairs, timeout = None, vhost = ''):
        "Set multiple values on multiple keys"
        if not kvpairs:
            raise StopIteration
        c = self._redis_clients.get(vhost)
        d = kvpairs
        if hasattr(d, 'items'):
            d = d.items()
        if timeout is not None and timeout <= 0:
            for m in c.execute_command(self.apiroutine, 'DEL', *[k for k,_ in d]):
                yield m
        else:
            if timeout is None:
                for m in c.execute_command(self.apiroutine, 'MSET', *list(itertools.chain.from_iterable(d))):
                    yield m
            else:
                # Use a transact
                ptimeout = int(timeout * 1000)
                
                for m in c.batch_execute(self.apiroutine, *((('MULTI',),
                                                             ('MSET',) + tuple(itertools.chain.from_iterable(d))) + \
                                                            tuple(('PEXPIRE', k, ptimeout) for k, _ in d) + \
                                                            (('EXEC',),))
                                                        ):
                    yield m
        self.apiroutine.retvalue = None
    def update(self, key, updater, timeout = None, vhost = ''):
        '''
        Update in-place with a custom function
        
        :param key: key to update
        
        :param updater: func(k,v), should return a new value to update, or return None to delete. The function
                        may be call more than once.
        
        :param timeout: new timeout
        
        :returns: the updated value, or None if deleted
        '''
        c = self._redis_clients.get(vhost)
        for m in c.get_connection(self.apiroutine):
            yield m
        newconn = self.apiroutine.retvalue
        spin = self.maxspin
        with newconn.context(self.apiroutine):
            for i in range(0, self.maxretry):
                for m in newconn.execute_command(self.apiroutine, 'WATCH', key):
                    yield m
                for m in newconn.execute_command(self.apiroutine, 'GET', key):
                    yield m
                v = self._decode(self.apiroutine.retvalue)
                try:
                    v = updater(key, v)
                    if v is not None:
                        ve = self._encode(v)
                except Exception as exc:
                    for m in newconn.execute_command(self.apiroutine, 'UNWATCH'):
                        yield m
                    raise exc
                if timeout is not None and timeout <= 0 or v is None:
                    set_command = ('DEL', key)
                elif timeout is None:
                    set_command = ('SET', key, ve)
                else:
                    set_command = ('PSETEX', key, int(timeout * 1000), ve)
                for m in newconn.batch_execute(self.apiroutine, ('MULTI',),
                                                                set_command,
                                                                ('EXEC',)):
                    yield m
                r = self.apiroutine.retvalue[2]
                if r is not None:
                    # Succeeded
                    self.apiroutine.retvalue = v
                    raise StopIteration
                else:
                    # Watch keys changed, retry
                    if i > spin:
                        # Wait for a random small while
                        for m in self.apiroutine.waitWithTimeout(random.randrange(0, 1<<(i-spin)) * 0.05):
                            yield m
            raise RedisWriteConflictException('Transaction still fails after many retries: key=' + repr(key))
                
    def mupdate(self, keys, updater, timeout = None, vhost = ''):
        "Update multiple keys in-place with a custom function, see update. Either all success, or all fail."
        if not keys:
            self.apiroutine.retvalue = []
            raise StopIteration
        c = self._redis_clients.get(vhost)
        for m in c.get_connection(self.apiroutine):
            yield m
        newconn = self.apiroutine.retvalue
        spin = self.maxspin
        with newconn.context(self.apiroutine):
            for i in range(0, self.maxretry):
                for m in newconn.execute_command(self.apiroutine, 'WATCH', *keys):
                    yield m
                for m in newconn.execute_command(self.apiroutine, 'MGET', *keys):
                    yield m
                values = self.apiroutine.retvalue
                try:
                    values = [updater(k, self._decode(v)) for k,v in izip(keys,values)]
                    keys_deleted = [k for k,v in izip(keys,values) if v is None]
                    values_encoded = [(k,self._encode(v)) for k,v in izip(keys,values) if v is not None]
                except Exception as exc:
                    for m in newconn.execute_command(self.apiroutine, 'UNWATCH'):
                        yield m
                    raise exc
                if timeout is None:
                    set_commands_list = []
                    if values_encoded:
                        set_commands_list.append(('MSET',) + tuple(itertools.chain.from_iterable(values_encoded)))
                    if keys_deleted:
                        set_commands_list.append(('DEL',) + tuple(keys_deleted))
                    set_commands = tuple(set_commands_list)
                elif timeout <= 0:
                    set_commands = (('DEL',) + tuple(keys),)
                else:
                    ptimeout = int(timeout * 1000)
                    set_commands_list = []
                    if values_encoded:
                        set_commands_list.append(('MSET',) + tuple(itertools.chain.from_iterable(values_encoded)))
                        set_commands_list.extend(('PEXPIRE', k, ptimeout) for k,_ in values_encoded)
                    if keys_deleted:
                        set_commands_list.append(('DEL',) + tuple(keys_deleted))
                    set_commands = tuple(set_commands_list)
                for m in newconn.batch_execute(self.apiroutine, *((('MULTI',),) + \
                                                                set_commands + \
                                                                (('EXEC',),))):
                    yield m
                r = self.apiroutine.retvalue[-1]
                if r is not None:
                    # Succeeded
                    self.apiroutine.retvalue = values
                    raise StopIteration
                else:
                    # Watch keys changed, retry
                    if i > spin:
                        # Wait for a random small while
                        for m in self.apiroutine.waitWithTimeout(random.randrange(0, 1<<(i-spin)) * 0.05):
                            yield m
            raise RedisWriteConflictException('Transaction still fails after many retries: keys=' + repr(keys))
    def updateall(self, keys, updater, timeout = None, vhost = ''):
        "Update multiple keys in-place, with a function updater(keys, values) which returns (updated_keys, updated_values). Either all success or all fail"
        c = self._redis_clients.get(vhost)
        for m in c.get_connection(self.apiroutine):
            yield m
        newconn = self.apiroutine.retvalue
        spin = self.maxspin
        with newconn.context(self.apiroutine):
            for i in range(0, self.maxretry):
                if keys:
                    for m in newconn.execute_command(self.apiroutine, 'WATCH', *keys):
                        yield m
                    for m in newconn.execute_command(self.apiroutine, 'MGET', *keys):
                        yield m
                    values = self.apiroutine.retvalue
                else:
                    values = []
                try:
                    new_keys, new_values = updater(keys, [self._decode(v) for v in values])
                    keys_deleted = [k for k,v in izip(new_keys, new_values) if v is None]
                    values_encoded = [(k,self._encode(v)) for k,v in izip(new_keys, new_values) if v is not None]
                except Exception as exc:
                    if keys:
                        for m in newconn.execute_command(self.apiroutine, 'UNWATCH'):
                            yield m
                    raise exc
                if timeout is None:
                    set_commands_list = []
                    if values_encoded:
                        set_commands_list.append(('MSET',) + tuple(itertools.chain.from_iterable(values_encoded)))
                    if keys_deleted:
                        set_commands_list.append(('DEL',) + tuple(keys_deleted))
                    set_commands = tuple(set_commands_list)
                elif timeout <= 0:
                    if new_keys:
                        set_commands = (('DEL',) + tuple(new_keys),)
                    else:
                        set_commands = ()
                else:
                    ptimeout = int(timeout * 1000)
                    set_commands_list = []
                    if values_encoded:
                        set_commands_list.append(('MSET',) + tuple(itertools.chain.from_iterable(values_encoded)))
                        set_commands_list.extend(('PEXPIRE', k, ptimeout) for k,_ in values_encoded)
                    if keys_deleted:
                        set_commands_list.append(('DEL',) + tuple(keys_deleted))
                    set_commands = tuple(set_commands_list)
                for m in newconn.batch_execute(self.apiroutine, *((('MULTI',),) + \
                                                                set_commands + \
                                                                (('EXEC',),))):
                    yield m
                r = self.apiroutine.retvalue[-1]
                if r is not None:
                    # Succeeded
                    self.apiroutine.retvalue = values
                    raise StopIteration
                else:
                    # Watch keys changed, retry
                    if i > spin:
                        # Wait for a random small while
                        for m in self.apiroutine.waitWithTimeout(random.randrange(0, 1<<(i-spin)) * 0.05):
                            yield m
            raise RedisWriteConflictException('Transaction still fails after many retries: keys=' + repr(keys))
    def updateallwithtime(self, keys, updater, timeout = None, vhost = ''):
        "Update multiple keys in-place, with a function updater(keys, values, timestamp) which returns (updated_keys, updated_values). Either all success or all fail. Timestamp is a integer standing for current time in microseconds."
        c = self._redis_clients.get(vhost)
        for m in c.get_connection(self.apiroutine):
            yield m
        newconn = self.apiroutine.retvalue
        spin = self.maxspin
        with newconn.context(self.apiroutine):
            for i in range(0, self.maxretry):
                if keys:
                    for m in newconn.execute_command(self.apiroutine, 'WATCH', *keys):
                        yield m
                    for m in newconn.batch_execute(self.apiroutine, ('MGET',) + tuple(keys),
                                                                    ('TIME',)):
                        yield m
                    values, time_tuple = self.apiroutine.retvalue
                else:
                    for m in newconn.execute_command(self.apiroutine, 'TIME'):
                        yield m
                    values = []
                    time_tuple = self.apiroutine.retvalue
                server_time = int(time_tuple[0]) * 1000000 + int(time_tuple[1])
                try:
                    new_keys, new_values = updater(keys, [self._decode(v) for v in values], server_time)
                    keys_deleted = [k for k,v in izip(new_keys, new_values) if v is None]
                    values_encoded = [(k,self._encode(v)) for k,v in izip(new_keys, new_values) if v is not None]
                except Exception as exc:
                    if keys:
                        for m in newconn.execute_command(self.apiroutine, 'UNWATCH'):
                            yield m
                    raise exc
                if timeout is None:
                    set_commands_list = []
                    if values_encoded:
                        set_commands_list.append(('MSET',) + tuple(itertools.chain.from_iterable(values_encoded)))
                    if keys_deleted:
                        set_commands_list.append(('DEL',) + tuple(keys_deleted))
                    set_commands = tuple(set_commands_list)
                elif timeout <= 0:
                    if new_keys:
                        set_commands = (('DEL',) + tuple(new_keys),)
                    else:
                        set_commands = ()
                else:
                    ptimeout = int(timeout * 1000)
                    set_commands_list = []
                    if values_encoded:
                        set_commands_list.append(('MSET',) + tuple(itertools.chain.from_iterable(values_encoded)))
                        set_commands_list.extend(('PEXPIRE', k, ptimeout) for k,_ in values_encoded)
                    if keys_deleted:
                        set_commands_list.append(('DEL',) + tuple(keys_deleted))
                    set_commands = tuple(set_commands_list)
                for m in newconn.batch_execute(self.apiroutine, *((('MULTI',),) + \
                                                                set_commands + \
                                                                (('EXEC',),))):
                    yield m
                r = self.apiroutine.retvalue[-1]
                if r is not None:
                    # Succeeded
                    self.apiroutine.retvalue = values
                    raise StopIteration
                else:
                    # Watch keys changed, retry
                    if i > spin:
                        # Wait for a random small while
                        for m in self.apiroutine.waitWithTimeout(random.randrange(0, 1<<(i-spin)) * 0.05):
                            yield m
            raise RedisWriteConflictException('Transaction still fails after many retries: keys=' + repr(keys))
    


'''
Created on 2015/7/20/

:author: hubo
'''
from __future__ import print_function
import unittest
from vlcp.protocol.openflow import common, openflow10, openflow13
from namedstruct import nstruct, dump
from pprint import pprint
import vlcp.utils.ethernet as ethernet

class Test(unittest.TestCase):
    exclude = [common.ofp_error_experimenter_msg, openflow13.ofp_group_desc_stats, openflow13.ofp_oxm_mask, openflow13.ofp_oxm_nomask, openflow13._ofp_oxm_mask_value,
               openflow13.ofp_action_set_field, openflow10.nx_flow_mod_spec, openflow13.nx_flow_mod_spec, openflow10.nx_matches, openflow13.nx_matches]
    def testDefs10(self):
        for k in dir(openflow10):
            attr = getattr(openflow10, k)
            if isinstance(attr, nstruct) and not attr in self.exclude and not k.startswith('nxm_') and not hasattr(ethernet, k):
                if not attr.subclasses:
                    self.assertEqual(k, repr(attr), k + ' has different name: ' + repr(attr))
                    print(k, repr(attr))
                    obj = attr.new()
                    s = obj._tobytes()
                    r = attr.parse(s)
                    self.assertTrue(r is not None, repr(attr) + ' failed to parse')
                    obj2, size = r
                    self.assertEqual(size, len(s), repr(attr) + ' failed to parse')
                    self.assertEqual(dump(obj), dump(obj2), repr(attr) + ' changed after parsing')
    def testDefs13(self):
        for k in dir(openflow13):
            attr = getattr(openflow13, k)
            if isinstance(attr, nstruct) and not attr in self.exclude and not k.startswith('ofp_oxm_') and not k.startswith('nxm_') and not hasattr(ethernet, k):
                if not attr.subclasses:
                    self.assertEqual(k, repr(attr), k + ' has different name: ' + repr(attr))
                    print(k, repr(attr))
                    obj = attr.new()
                    s = obj._tobytes()
                    r = attr.parse(s)
                    self.assertTrue(r is not None, repr(attr) + ' failed to parse')
                    obj2, size = r
                    self.assertEqual(size, len(s), repr(attr) + ' failed to parse')
                    self.assertEqual(dump(obj), dump(obj2), repr(attr) + ' changed after parsing')
    def testOxm(self):
        fm = openflow13.ofp_flow_mod.new(priority = openflow13.OFP_DEFAULT_PRIORITY, command = openflow13.OFPFC_ADD, buffer_id = openflow13.OFP_NO_BUFFER)
        fm.cookie = 0x67843512
        fm.match = openflow13.ofp_match_oxm.new()
        fm.match.oxm_fields.append(openflow13.create_oxm(openflow13.OXM_OF_ETH_DST, b'\x06\x00\x0c\x15\x45\x99'))
        fm.match.oxm_fields.append(openflow13.create_oxm(openflow13.OXM_OF_ETH_TYPE, common.ETHERTYPE_IP))
        fm.match.oxm_fields.append(openflow13.create_oxm(openflow13.OXM_OF_IP_PROTO, 6))
        fm.match.oxm_fields.append(openflow13.create_oxm(openflow13.OXM_OF_IPV4_SRC_W, [192,168,1,0], [255,255,255,0]))
        apply = openflow13.ofp_instruction_actions.new(type = openflow13.OFPIT_APPLY_ACTIONS)
        apply.actions.append(openflow13.ofp_action_set_field.new(field = openflow13.create_oxm(openflow13.OXM_OF_IPV4_SRC, [202, 102, 0, 37])))
        apply.actions.append(openflow13.ofp_action_set_queue.new(queue_id = 1))
        fm.instructions.append(apply)
        write = openflow13.ofp_instruction_actions.new(type = openflow13.OFPIT_WRITE_ACTIONS)
        write.actions.append(openflow13.ofp_action_output.new(port = 7))
        fm.instructions.append(write)
        goto = openflow13.ofp_instruction_goto_table.new(table_id = 1)
        fm.instructions.append(goto)
        s = fm._tobytes()
        r = common.ofp_msg.parse(s)
        self.assertTrue(r is not None, 'Cannot parse message')
        obj2, size = r
        self.assertEqual(size, len(s), 'Cannot parse message')
        pprint(dump(fm))
        pprint(dump(obj2))
        self.assertEqual(dump(fm), dump(obj2), 'message changed after parsing')
    def testDefs13Size(self):
        # From openflow.h
        self.assertEqual(len(openflow13.ofp_header()),8)
        self.assertEqual(openflow13.ofp_hello_elem()._realsize(),4)        # Excluding padding
        self.assertEqual(openflow13.ofp_hello_elem_versionbitmap()._realsize(),4)
        self.assertEqual(len(openflow13.ofp_hello()),8)
        self.assertEqual(len(openflow13.ofp_switch_config()),12)
        self.assertEqual(len(openflow13.ofp_table_mod()),16)
        self.assertEqual(len(openflow13.ofp_port()),64)
        self.assertEqual(len(openflow13.ofp_switch_features()),32)
        self.assertEqual(len(openflow13.ofp_port_status()),80)
        self.assertEqual(len(openflow13.ofp_port_mod()),40)
        self.assertEqual(len(openflow13.ofp_match()),8)
        self.assertEqual(len(openflow13.ofp_oxm_experimenter()),8)
        self.assertEqual(len(openflow13.ofp_action()),8)
        self.assertEqual(len(openflow13.ofp_action_output()),16)
        self.assertEqual(len(openflow13.ofp_action_mpls_ttl()),8)
        self.assertEqual(len(openflow13.ofp_action_push()),8)
        self.assertEqual(len(openflow13.ofp_action_pop_mpls()),8)
        self.assertEqual(len(openflow13.ofp_action_group()),8)
        self.assertEqual(len(openflow13.ofp_action_nw_ttl()),8)
        self.assertEqual(len(openflow13.ofp_action_set_field()),8)
        self.assertEqual(len(openflow13.ofp_action_experimenter()),8)
        self.assertEqual(openflow13.ofp_instruction()._realsize(),4)
        self.assertEqual(len(openflow13.ofp_instruction_goto_table()),8)
        self.assertEqual(len(openflow13.ofp_instruction_write_metadata()),24)
        self.assertEqual(len(openflow13.ofp_instruction_actions()),8)
        self.assertEqual(len(openflow13.ofp_instruction_meter()),8)
        self.assertEqual(len(openflow13.ofp_instruction_experimenter()),8)
        self.assertEqual(len(openflow13.ofp_flow_mod()),56)
        self.assertEqual(len(openflow13.ofp_bucket()),16)
        self.assertEqual(len(openflow13.ofp_group_mod()),16)
        self.assertEqual(len(openflow13.ofp_packet_out()),24)
        self.assertEqual(len(openflow13.ofp_packet_in()),34)    # Add the extra padding
        self.assertEqual(len(openflow13.ofp_flow_removed()),56)
        self.assertEqual(openflow13.ofp_meter_band()._realsize(),12)
        self.assertEqual(len(openflow13.ofp_meter_band_drop()),16)
        self.assertEqual(len(openflow13.ofp_meter_band_dscp_remark()),16)
        self.assertEqual(len(openflow13.ofp_meter_band_experimenter()),16)
        self.assertEqual(len(openflow13.ofp_meter_mod()),16)
        self.assertEqual(len(openflow13.ofp_error_msg()),12)
        self.assertEqual(len(openflow13.ofp_error_experimenter_msg()),16)
        self.assertEqual(len(openflow13.ofp_multipart_request()),16)
        self.assertEqual(len(openflow13.ofp_multipart_reply()),16)
        self.assertEqual(len(openflow13.ofp_desc()),1056)
        self.assertEqual(len(openflow13.ofp_flow_stats_request()),40 + len(openflow13.ofp_multipart_request()))
        self.assertEqual(len(openflow13.ofp_flow_stats()),56)
        self.assertEqual(len(openflow13.ofp_aggregate_stats_request()),40 + len(openflow13.ofp_multipart_request()))
        self.assertEqual(len(openflow13.ofp_aggregate_stats_reply()),24 + len(openflow13.ofp_multipart_reply()))
        self.assertEqual(openflow13.ofp_table_feature_prop()._realsize(),4)
        self.assertEqual(openflow13.ofp_table_feature_prop_instructions()._realsize(),4)
        self.assertEqual(openflow13.ofp_table_feature_prop_next_tables()._realsize(),4)
        self.assertEqual(openflow13.ofp_table_feature_prop_actions()._realsize(),4)
        self.assertEqual(openflow13.ofp_table_feature_prop_oxm()._realsize(),4)
        self.assertEqual(openflow13.ofp_table_feature_prop_experimenter()._realsize(),12)
        self.assertEqual(len(openflow13.ofp_table_features()),64)
        self.assertEqual(len(openflow13.ofp_table_stats()),24)
        self.assertEqual(len(openflow13.ofp_port_stats_request()),8 + len(openflow13.ofp_multipart_request()))
        self.assertEqual(len(openflow13.ofp_port_stats()),112)
        self.assertEqual(len(openflow13.ofp_group_stats_request()),8 + len(openflow13.ofp_multipart_request()))
        self.assertEqual(len(openflow13.ofp_bucket_counter()),16)
        self.assertEqual(len(openflow13.ofp_group_stats()),40)
        self.assertEqual(len(openflow13.ofp_group_desc()),8)
        self.assertEqual(len(openflow13.ofp_group_features()),40)
        self.assertEqual(len(openflow13.ofp_meter_multipart_request()),8 + len(openflow13.ofp_multipart_request()))
        self.assertEqual(len(openflow13.ofp_meter_band_stats()),16)
        self.assertEqual(len(openflow13.ofp_meter_stats()),40)
        self.assertEqual(len(openflow13.ofp_meter_config()),8 + len(openflow13.ofp_multipart_reply()))
        self.assertEqual(len(openflow13.ofp_meter_features()),16)
        self.assertEqual(len(openflow13.ofp_experimenter_multipart_header()),8)
        self.assertEqual(len(openflow13.ofp_experimenter()),16)
        self.assertEqual(len(openflow13.ofp_queue_prop_header()),8)
        self.assertEqual(len(openflow13.ofp_queue_prop_min_rate()),16)
        self.assertEqual(len(openflow13.ofp_queue_prop_max_rate()),16)
        self.assertEqual(len(openflow13.ofp_queue_prop_experimenter()),16)
        self.assertEqual(len(openflow13.ofp_packet_queue()),16)
        self.assertEqual(len(openflow13.ofp_queue_get_config_request()),16)
        self.assertEqual(len(openflow13.ofp_queue_get_config_reply()),16)
        self.assertEqual(len(openflow13.ofp_action_set_queue()),8)
        self.assertEqual(len(openflow13.ofp_queue_stats_request()),8 + len(openflow13.ofp_multipart_reply()))
        self.assertEqual(len(openflow13.ofp_queue_stats()),40)
        self.assertEqual(len(openflow13.ofp_role_request()),24)
        self.assertEqual(len(openflow13.ofp_async_config()),32)
    def testDefs10Size(self):
        self.assertEqual(len(openflow10.ofp_header()),8)
        self.assertEqual(len(openflow10.ofp_phy_port()),48)
        self.assertEqual(len(openflow10.ofp_packet_queue()),8)
        self.assertEqual(len(openflow10.ofp_queue_prop_header()),8)
        self.assertEqual(len(openflow10.ofp_queue_prop_min_rate()),16)
        self.assertEqual(len(openflow10.ofp_match()),40)
        self.assertEqual(len(openflow10.ofp_action()),8)
        self.assertEqual(len(openflow10.ofp_action_output()),8)
        self.assertEqual(len(openflow10.ofp_action_enqueue()),16)
        self.assertEqual(len(openflow10.ofp_action_vlan_vid()),8)
        self.assertEqual(len(openflow10.ofp_action_vlan_pcp()),8)
        self.assertEqual(len(openflow10.ofp_action_dl_addr()),16)
        self.assertEqual(len(openflow10.ofp_action_nw_addr()),8)
        self.assertEqual(len(openflow10.ofp_action_nw_tos()),8)
        self.assertEqual(len(openflow10.ofp_action_tp_port()),8)
        self.assertEqual(len(openflow10.ofp_action_vendor()),8)
        self.assertEqual(len(openflow10.ofp_switch_features()),32)
        self.assertEqual(len(openflow10.ofp_switch_config()),12)
        self.assertEqual(len(openflow10.ofp_flow_mod()),72)
        self.assertEqual(len(openflow10.ofp_port_mod()),32)
        self.assertEqual(len(openflow10.ofp_queue_get_config_request()),12)
        self.assertEqual(len(openflow10.ofp_queue_get_config_reply()),16)
        self.assertEqual(len(openflow10.ofp_stats_request()),12)
        self.assertEqual(len(openflow10.ofp_stats_reply()),12)
        self.assertEqual(len(openflow10.ofp_desc_stats()),1056)
        self.assertEqual(len(openflow10.ofp_flow_stats_request()),44 + len(openflow10.ofp_stats_request()))
        self.assertEqual(len(openflow10.ofp_flow_stats()),88)
        self.assertEqual(len(openflow10.ofp_aggregate_stats_request()),44 + len(openflow10.ofp_stats_request()))
        self.assertEqual(len(openflow10.ofp_aggregate_stats_reply()),24 + len(openflow10.ofp_stats_reply()))
        self.assertEqual(len(openflow10.ofp_table_stats()),64)
        self.assertEqual(len(openflow10.ofp_port_stats_request()),8 + len(openflow10.ofp_stats_request()))
        self.assertEqual(len(openflow10.ofp_port_stats()),104)
        self.assertEqual(len(openflow10.ofp_queue_stats_request()),8 + len(openflow10.ofp_stats_request()))
        self.assertEqual(len(openflow10.ofp_queue_stats()),32)
        self.assertEqual(len(openflow10.ofp_packet_out()),16)
        self.assertEqual(len(openflow10.ofp_packet_in()),18)        # No extra padding
        self.assertEqual(len(openflow10.ofp_flow_removed()),88)
        self.assertEqual(len(openflow10.ofp_port_status()),64)
        self.assertEqual(len(openflow10.ofp_error_msg()),12)
        self.assertEqual(len(openflow10.ofp_vendor()),12)
    def testDefs10ExtSize(self):
        self.assertEqual(len(openflow10.nicira_header()),16)
        self.assertEqual(len(openflow10.nx_stats_request()),24)
        self.assertEqual(len(openflow10.nx_flow_mod_table_id()),8 + len(openflow10.nicira_header()))
        self.assertEqual(len(openflow10.nx_set_packet_in_format()),4 + len(openflow10.nicira_header()))
        self.assertEqual(len(openflow10.nx_packet_in()),24 + len(openflow10.nicira_header()) + 2) # Extra padding
        self.assertEqual(len(openflow10.nx_role_request()),4 + len(openflow10.nicira_header()))
        self.assertEqual(len(openflow10.nx_async_config()),24 + len(openflow10.nicira_header()))
        self.assertEqual(len(openflow10.nx_set_flow_format()),4 + len(openflow10.nicira_header()))
        self.assertEqual(len(openflow10.nx_flow_mod()),32 + len(openflow10.nicira_header()))
        self.assertEqual(len(openflow10.nx_flow_removed()),40 + len(openflow10.nicira_header()))
        self.assertEqual(len(openflow10.nx_flow_stats_request()),8 + len(openflow10.nx_stats_request()))
        self.assertEqual(len(openflow10.nx_flow_stats()),48)
        self.assertEqual(len(openflow10.nx_aggregate_stats_request()),8 + len(openflow10.nx_stats_request()))
        self.assertEqual(len(openflow10.nx_controller_id()),8 + len(openflow10.nicira_header()))
        self.assertEqual(len(openflow10.nx_flow_monitor_request()),16 + len(openflow10.nx_stats_request()))
        self.assertEqual(openflow10.nx_flow_update()._realsize(),4)
        self.assertEqual(len(openflow10.nx_flow_update_full()),24)
        self.assertEqual(len(openflow10.nx_flow_update_abbrev()),8)
        self.assertEqual(len(openflow10.nx_flow_monitor_cancel()),4 + len(openflow10.nicira_header()))
    def testDefs13ExtSize(self):
        self.assertEqual(len(openflow13.nicira_header()),16)
        self.assertEqual(len(openflow13.nx_stats_request()),24)
        self.assertEqual(len(openflow13.nx_flow_mod_table_id()),8 + len(openflow13.nicira_header()))
        self.assertEqual(len(openflow13.nx_set_packet_in_format()),4 + len(openflow13.nicira_header()))
        self.assertEqual(len(openflow13.nx_packet_in()),24 + len(openflow13.nicira_header()) + 2) # Extra padding
        self.assertEqual(len(openflow13.nx_role_request()),4 + len(openflow13.nicira_header()))
        self.assertEqual(len(openflow13.nx_async_config()),24 + len(openflow13.nicira_header()))
        self.assertEqual(len(openflow13.nx_set_flow_format()),4 + len(openflow13.nicira_header()))
        self.assertEqual(len(openflow13.nx_flow_mod()),32 + len(openflow13.nicira_header()))
        self.assertEqual(len(openflow13.nx_flow_removed()),40 + len(openflow13.nicira_header()))
        self.assertEqual(len(openflow13.nx_flow_stats_request()),8 + len(openflow13.nx_stats_request()))
        self.assertEqual(len(openflow13.nx_flow_stats()),48)
        self.assertEqual(len(openflow13.nx_aggregate_stats_request()),8 + len(openflow13.nx_stats_request()))
        self.assertEqual(len(openflow13.nx_controller_id()),8 + len(openflow13.nicira_header()))
        self.assertEqual(len(openflow13.nx_flow_monitor_request()),16 + len(openflow13.nx_stats_request()))
        self.assertEqual(openflow13.nx_flow_update()._realsize(),4)
        self.assertEqual(len(openflow13.nx_flow_update_full()),24)
        self.assertEqual(len(openflow13.nx_flow_update_abbrev()),8)
        self.assertEqual(len(openflow13.nx_flow_monitor_cancel()),4 + len(openflow13.nicira_header()))
if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testDefs']
    unittest.main()



from vlcp.server.module import Module, api
from vlcp.config import defaultconfig
from vlcp.event import Event, withIndices
from vlcp.event.runnable import RoutineContainer

@withIndices()
class ModuleTestEvent(Event):
    pass

@defaultconfig
class TestModule1(Module):
    class MyHandler(RoutineContainer):
        def method2(self, a, b):
            self.retvalue = a + b
            if False:
                yield
        def method3(self, a, b):
            for m in self.waitForSend(ModuleTestEvent(a = a, b = b)):
                yield m
            self.retvalue = None
    def __init__(self, server):
        Module.__init__(self, server)
        self.handlerRoutine = self.MyHandler(self.scheduler)
        self.createAPI(api(self.method1),
                api(self.handlerRoutine.method2,self.handlerRoutine),
                api(self.handlerRoutine.method3,self.handlerRoutine),
                api(self.method4)
                )
    def method1(self):
        return 'version3'
    def method4(self):
        raise ValueError('test')




from vlcp.server.module import Module, api, depend
from vlcp.config import defaultconfig
from vlcp.event import Event, withIndices
from vlcp.event.runnable import RoutineContainer
from . import testmodule1

@withIndices()
class ModuleTestEvent2(Event):
    pass

@defaultconfig
@depend(testmodule1.TestModule1)
class TestModule2(Module):
    class MyHandler(RoutineContainer):
        def main(self):
            matcher = testmodule1.ModuleTestEvent.createMatcher()
            while True:
                yield (matcher,)
                self.subroutine(self.waitForSend(ModuleTestEvent2(result=self.event.a + self.event.b, version = 'version3')), False)
    def __init__(self, server):
        Module.__init__(self, server)
        self.routines.append(self.MyHandler(self.scheduler))



'''
Created on 2015/7/8

:author: hubo
'''
import unittest
from vlcp.config import manager, Configurable, defaultconfig
from vlcp.protocol import Protocol

@defaultconfig
class TestConfigurable(Configurable):
    _default_testproperty = '123'

@defaultconfig
class TestSubClass(TestConfigurable):
    _default_testproperty = '456'
    testproperty2 = 123

@defaultconfig
class TestShortTestConfigurable(TestConfigurable):
    _default_testproperty = '456'
    testproperty2 = 123


class Test(unittest.TestCase):

    def testConfigurable(self):
        c1 = TestConfigurable()
        c2 = TestSubClass()
        c3 = TestShortTestConfigurable()
        self.assertEqual(getattr(c1, 'test', 'notconfigured'), 'notconfigured')
        self.assertEqual(getattr(c2, 'test', 'notconfigured'), 'notconfigured')
        manager['testconfigurable.default.test'] = 789
        self.assertEqual(getattr(c1, 'test', 'notconfigured'), 789)
        self.assertEqual(getattr(c2, 'test', 'notconfigured'), 789)
        manager['testconfigurable.testsubclass.test'] = 456
        self.assertEqual(getattr(c1, 'test', 'notconfigured'), 789)
        self.assertEqual(getattr(c2, 'test', 'notconfigured'), 456)
        self.assertEqual(getattr(c1, 'testproperty', 'notconfigured'), '123')
        self.assertEqual(getattr(c2, 'testproperty', 'notconfigured'), '456')
        manager['testconfigurable.default.testproperty'] = 111
        self.assertEqual(getattr(c1, 'testproperty', 'notconfigured'), 111)
        self.assertEqual(getattr(c2, 'testproperty', 'notconfigured'), '456')
        manager['testconfigurable.testsubclass.testproperty'] = 222
        self.assertEqual(getattr(c1, 'testproperty', 'notconfigured'), 111)
        self.assertEqual(getattr(c2, 'testproperty', 'notconfigured'), 222)
        self.assertEqual(getattr(c1, 'testproperty2', 'notconfigured'), 'notconfigured')
        self.assertEqual(getattr(c2, 'testproperty2', 'notconfigured'), 123)
        manager['testconfigurable.default.testproperty2'] = 321
        self.assertEqual(getattr(c1, 'testproperty2', 'notconfigured'), 321)
        self.assertEqual(getattr(c2, 'testproperty2', 'notconfigured'), 123)
        manager['testconfigurable.testsubclass.testproperty2'] = 333
        self.assertEqual(getattr(c1, 'testproperty2', 'notconfigured'), 321)
        self.assertEqual(getattr(c2, 'testproperty2', 'notconfigured'), 123)
        c1.testproperty = 777
        c2.testproperty = 999
        self.assertEqual(getattr(c1, 'testproperty', 'notconfigured'), 777)
        self.assertEqual(getattr(c2, 'testproperty', 'notconfigured'), 999)
        self.assertEqual(getattr(c3, 'test', 'notconfigured'), 789)
        manager['testconfigurable.testshorttestconfigurable.test'] = 888
        self.assertEqual(getattr(c3, 'test', 'notconfigured'), 888)
        self.assertEqual(manager.testconfigurable.testshorttestconfigurable.test, 888)
        self.assertEqual(manager.testconfigurable['testshorttestconfigurable.test'], 888)
        self.assertEqual(len(manager.testconfigurable), 3)
        self.assertEqual(set(manager.testconfigurable), set(['default', 'testsubclass', 'testshorttestconfigurable']))
        self.assertEqual(set(manager.testconfigurable.config_keys()), set(['default.test', 'testsubclass.test', 'default.testproperty',
                                                                              'testsubclass.testproperty', 'default.testproperty2',
                                                                              'testsubclass.testproperty2', 'testshorttestconfigurable.test']))
    def testConfigFile(self):
        manager.clear()
        manager['testa.testb.test1'] = 123
        manager['testa.testb.test2'] = "abc"
        manager['testa.testb.test3.test4'] = (123,"abc")
        manager['testa.testb.test3.test5'] = ['abc', u'def', (123,"abc")]
        manager['testa.testb.test3.test6'] = {'abc':123,b'def':u'ghi','jkl':[(123.12,345),"abc"]}
        save = manager.save()
        import os
        import tests
        os.chdir(tests.__path__[0])
        manager.saveto('../testconfigs/testconfig.cfg')
        manager.clear()
        manager.loadfrom('../testconfigs/testconfig.cfg')
        self.assertEqual(list(manager.config_items(True)), [('testa.testb.test1', 123),
                                                            ('testa.testb.test2', "abc"),
                                                            ('testa.testb.test3.test4', (123,"abc")),
                                                            ('testa.testb.test3.test5', ['abc', u'def', (123,"abc")]),
                                                            ('testa.testb.test3.test6', {'abc':123,b'def':u'ghi','jkl':[(123.12,345),"abc"]})])
        #save2 = manager.save()
        #self.assertEqual(save, save2)

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



'''
Created on 2015/8/27

:author: hubo
'''
from pprint import pprint
from vlcp.server import Server
from vlcp.event import RoutineContainer, Stream, TcpServer, MemoryStream
from vlcp.protocol.http import Http, HttpRequestEvent, escape_b, escape
from codecs import getincrementalencoder
import logging

http = Http(True)

class MainRoutine(RoutineContainer):
    def __init__(self, scheduler=None, daemon=False):
        RoutineContainer.__init__(self, scheduler=scheduler, daemon=daemon)
        self.encoder = getincrementalencoder('utf-8')
    def main(self):
        request = HttpRequestEvent.createMatcher()
        while True:
            yield (request,)            
            self.event.canignore = True
            self.subroutine(self.handlehttp(self.event))
    document = '''
<!DOCTYPE html >
<html>
<head>
<title>Test Server Page</title>
</head>
<body>
OK!<br/>
Host = %s<br/>
Path = %s<br/>
Headers = %s<br/>
</body>
</html>
'''
    def formatstr(self, tmpl, params):
        if not isinstance(tmpl, bytes):
            # Python 3
            return (tmpl % tuple(v.decode('utf-8') for v in params)).encode('utf-8')
        else:
            return (tmpl % params)
    def handlehttp(self, event):
        if event.stream is not None:
            event.stream.close(self.scheduler)
        if event.connmark == event.connection.connmark and event.connection.connected:
            #outputstream = Stream()
            document = self.formatstr(self.document, (escape_b(event.host), escape_b(event.path), escape(repr(event.headers)).encode('utf-8')))
            outputstream = MemoryStream(document)
            http.startResponse(event.connection, event.xid, b'200 OK', [], outputstream)
            #for m in outputstream.write(self.document, self, True):
                #yield m
            if False:
                yield

if __name__ == '__main__':
    logging.basicConfig()
    s = Server()
    #s.scheduler.debugging = True
    #s.scheduler.logger.setLevel(logging.DEBUG)
    #Http.debugging = True
    #Http._logger.setLevel(logging.DEBUG)
    http.createmessagequeue(s.scheduler)
    routine = MainRoutine(s.scheduler)
    routine.start()
    mgt_conn = TcpServer('ltcp://0.0.0.0:8080/', http, s.scheduler)
    mgt_conn.start()
    s.serve()
    
